@@ -1,8 +1,7 @@
 #ifndef _PROVALUE_H
 #define _PROVALUE_H	1
 
-ABSTRACT_VALUE* walk_through_nested_loops (struct tmplpro_param *param, PSTRING name);
-
+PSTRING get_variable_value (struct tmplpro_param *param, PSTRING name);
 
 #endif /* provalue.h */
 
